VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "TagParser"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Public Event ParseError(ByVal message As String)
Public Event InteriorTag(ByVal tag_name As String, ByVal tag_contents As String, ByVal depth As Integer)
Public Event LeafTag(ByVal tag_name As String, ByVal tag_contents As String, ByVal depth As Integer)
' Return True if the character is whitespace.
Private Function IsWhitespace(ByVal ch As String) As Boolean
    IsWhitespace = ( _
        ch = " " Or _
        ch = vbTab Or _
        ch = vbCr Or _
        ch = vbLf Or _
        ch = vbFormFeed Or _
        ch = vbNewLine Or _
        ch = vbVerticalTab)
End Function
' Parse the input text for tags.
Public Sub ParseTags(ByVal input_text As String)
    ParseTheTags input_text, 0
End Sub
' Pull off the next level of tag.
Private Sub ParseTheTags(ByRef input_text As String, ByVal depth As Integer)
Dim pos1 As Integer
Dim pos2 As Integer
Dim tag_name As String
Dim tag_contents As String

    ' Trim the text.
    input_text = TrimWhitespace(input_text)

    ' Repeat while there is input text left.
    Do While Len(input_text) > 0
        ' Make sure the text begins with a tag.
        If Left$(input_text, 1) <> "<" Then
            RaiseEvent ParseError( _
                "Expected '<' but found '" & _
                input_text & "'")
            Exit Sub
        End If

        ' Find the start tag.
        pos1 = InStr(input_text, ">")
        tag_name = Mid$(input_text, 2, pos1 - 2)

        ' Find the end tag.
        pos2 = InStr(input_text, "</" & tag_name & ">")
        tag_contents = TrimWhitespace(Mid$(input_text, pos1 + 1, pos2 - pos1 - 1))

        ' See if the tag_contents begins with a tag.
        If Left$(tag_contents, 1) = "<" Then
            ' This is an interior tag.
            ' Raise the InteriorTag event.
            RaiseEvent InteriorTag(tag_name, tag_contents, depth)

            ' Parse the tag for child tags.
            ParseTheTags tag_contents, depth + 1
        Else
            ' This is a leaf tag.
            ' Raise the LeafTag event.
            RaiseEvent LeafTag(tag_name, tag_contents, depth)
        End If

        ' Remove the tag from the input text.
        input_text = TrimWhitespace( _
            Mid$(input_text, pos2 + Len(tag_name) + 3))
    Loop
End Sub
' Remove whitespace fomr the from and back of the text.
Private Function TrimWhitespace(ByVal txt As String) As String
Dim pos1 As Integer
Dim pos2 As Integer
Dim ch As String

    ' Find the first non-whitespace character.
    For pos1 = 1 To Len(txt)
        If Not IsWhitespace(Mid$(txt, pos1, 1)) Then Exit For
    Next pos1

    ' Find the last non-whitespace character.
    For pos2 = Len(txt) To pos1 Step -1
        If Not IsWhitespace(Mid$(txt, pos2, 1)) Then Exit For
    Next pos2

    ' Return the result.
    If pos1 < pos1 Then
        TrimWhitespace = ""
    Else
        TrimWhitespace = Mid$(txt, pos1, pos2 - pos1 + 1)
    End If
End Function
